@@ -44,6 +44,7 @@ from lark.grammar import NonTerminal, Terminal
 
 import hypothesis._strategies as st
 from hypothesis.errors import InvalidArgument
+from hypothesis.internal.compat import getfullargspec
 from hypothesis.internal.conjecture.utils import calc_label_from_name
 from hypothesis.internal.validation import check_type
 from hypothesis.searchstrategy import SearchStrategy
@@ -78,9 +79,15 @@ class LarkStrategy(SearchStrategy):
         check_type(lark.lark.Lark, grammar, "grammar")
         if start is None:
             start = grammar.options.start
+        if not isinstance(start, list):
+            start = [start]
         self.grammar = grammar
 
-        terminals, rules, ignore_names = grammar.grammar.compile()
+        if "start" in getfullargspec(grammar.grammar.compile).args:
+            terminals, rules, ignore_names = grammar.grammar.compile(start)
+        else:  # pragma: no cover
+            # This branch is to support lark <= 0.7.1, without the start argument.
+            terminals, rules, ignore_names = grammar.grammar.compile()
 
         self.names_to_symbols = {}
 
@@ -91,10 +98,12 @@ class LarkStrategy(SearchStrategy):
         for t in terminals:
             self.names_to_symbols[t.name] = Terminal(t.name)
 
-        self.start = self.names_to_symbols[start]
+        self.start = st.sampled_from([self.names_to_symbols[s] for s in start])
 
-        self.ignored_symbols = st.sampled_from(
-            [self.names_to_symbols[n] for n in ignore_names]
+        self.ignored_symbols = (
+            st.sampled_from([self.names_to_symbols[n] for n in ignore_names])
+            if ignore_names
+            else st.nothing()
         )
 
         self.terminal_strategies = {
@@ -118,7 +127,8 @@ class LarkStrategy(SearchStrategy):
 
     def do_draw(self, data):
         state = DrawState()
-        self.draw_symbol(data, self.start, state)
+        start = data.draw(self.start)
+        self.draw_symbol(data, start, state)
         return u"".join(state.result)
 
     def rule_label(self, name):
